// Source file: brf/j2me/midp/dynaworks/Application.java
// Subsystem: dynaworks
// Module: Application

// Copyright (C) 2000 by Bernd R. Fix.
// You may distribute this software under the terms of the Modified
// Artistic Licence, as specified in the accompanying LICENCE.TXT file.

package brf.j2me.midp.dynaworks;

// import external definitions
import brf.j2me.midp.dynaworks.events.*;
import java.util.Hashtable;
import java.util.Vector;
import javax.microedition.midlet.MIDlet;
import javax.microedition.lcdui.Display;

/**
 * The Application class is the abstract base class for all DynaWorks-based
 * MIDP applications. It manages one or more Page objects that represent the
 * different views of the application.<p>
 *
 * @author Bernd R. Fix
 * @version 1.0
 * @see brf.j2me.midp.dynaworks.Page
 */
public class Application extends MIDlet implements DataStore {

    ///////////////////////////////////////////////////////////////////////////
    /*
     * inner class for the PageListener interface.<p>
     */
    public class PageHandler implements PageListener {
        
        /**
         * active page is closing.<p>
         *
         * @param nextPage String - name of next page to be displayed.
         */
        public void closing (String nextPage) {

            // handle the next page.
            activePage = nextPage;
            
            // did the last page just terminate?
            if (activePage == null || activePage.length() == 0) {		
                // yes: try pop the call stack.
                // is the stack empty?
                int pos = callstack.size() - 1;
                if (pos >= 0) {
                    // pop an entry from the stack.
                    activePage = (String) callstack.elementAt (pos);
                    callstack.removeElementAt (pos);
                    // intended fall thru...
                }
            }
            // are we subcalling on the next page?
            else if (activePage.charAt(0) == '@') {
                // yes: push the return name onto the stack.
                callstack.addElement (lastPage);
                activePage = activePage.substring (1);
            }

            // initiate a garbage collection.
            System.gc();
            
            // do we have a new page?
            if (activePage != null && activePage.length() > 0) {
                // show it.
                showPage ();
                return;
            }
            
            // if we have no more pages to render, we are done...
            destroyApp (false);
            notifyDestroyed ();
        }
    }
    
    //////////////////////////////////////////////////////////////////////
    // attributes
    /**
       Event queue for this application.<p>
     */
    private EventQueue events;
    /**
       list of pages managed by this application<p>
     */
    private Hashtable pages;
    /**
       object store for data used by all pages of this app.<p>
     */
    private Hashtable datastore;
    private Vector persistentVars;
    /**
       stack for nested pages<p>
     */
    private Vector callstack;
    /**
       Reference to active page<p>
     */
    private Page active;
    /**
     * name of active page.<p>
     */
    private String activePage;
    private String lastPage;
    /**
     * page handler.
     */
    protected PageListener handler;
    /**
     * Active screen to work on.<p>
     */
    private Display display;

    ///////////////////////////////////////////////////////////////////////////
    /**
     * constructor.<p>
     *
     * @param name String - name of the application.
     */
    public Application (String name) {

        // initialize attributes.
        datastore = new Hashtable ();      
        persistentVars = new Vector();
        pages = new Hashtable();
        callstack = new Vector();
        events = new EventQueue();
        active = null;
        activePage = null;
        handler = new PageHandler ();
        display = Display.getDisplay (this);
    }

    //////////////////////////////////////////////////////////////////////
    // page related methods.
    /**
     * add a Page object to the list.<p>
     *
     * @param p Page - page object to be added to list.
     * @return boolean - status of operation
     * @see brf.j2me.midp.dynaworks.Page
     * @see #removePage
     */
    public boolean addPage (Page p) {

        // get the name of the instance.
        String name = p.getName();
        
        // if this is the first page, remember it.
        if (activePage == null)
            activePage = name;
        
        // add the combination to the list if it's not already there.
        if (!pages.containsKey (name)) {
            // add to list of pages.
            pages.put (name, p);
            // set a back reference to the data dict.
            p.setDataStore (this);
            return true;
        }
        // nothing added.
        return false;
    }
    
    //---------------------------------------------------------
    /**
     * add a Page class to the list.<p>
     *
     * @param name String - name of the page
     * @param classN String - classname of the page
     * @return boolean - status of operation
     * @see brf.j2me.midp.dynaworks.Page
     * @see #removePage
     */
    public boolean addPage (String name, String classN) {

        // if this is the first page, remember it.
        if (activePage == null)
            activePage = name;
        
        // add the combination to the list if it's not already there.
        if (!pages.containsKey (name)) {
            // add to list of pages.
            pages.put (name, classN);
            return true;
        }
        // nothing added.
        return false;
    }
    
    //---------------------------------------------------------
    /**
     * remove a Page object to the list.<p>
     *
     * @param p Page - page object to be removed from list.
     * @return boolean - status of the operation.
     * @see brf.j2me.midp.dynaworks.Page
     * @see #addPage
     */
    public boolean removePage (Page p) {
        // get the name of the instance.
        String name = p.getName();	
        // blank out the data dict reference.
        p.setDataStore (null);
        // remove it.
        return removePage (name);
    }
    
    //---------------------------------------------------------
    /**
     *  remove a Page object to the list by name.<p>
     *
     * @param name String - name of the page object to be removed from list.
     * @return boolean - status of the operation.
     * @see brf.j2me.midp.dynaworks.Page
     * @see #addPage
     */
    public boolean removePage (String name) {
        // add the combination to the list if it's not already there.
        if (!pages.containsKey (name))
            return false;
        // try to remove object.
        return (pages.remove (name) != null);
    }
    
    //---------------------------------------------------------
    /**
     * set a startup page<p>
     *
     * @param name String - name of the startup page.
     * @see brf.j2me.midp.dynaworks.Page
     * @see #addPage
     */
    public void run (String name) {
        // remember the name
        activePage = name;
    }
    
    ///////////////////////////////////////////////////////////////////////////
    /**
     * instanciate a Page object by name (class factory).<p>
     *
     * @param name String - classname of the page object.
     * @return Page - instanciated page object.
     */
    protected Page instanciatePage (String name) {
        // return no object (not implemented yet)
        return null;
    }

    ///////////////////////////////////////////////////////////////////////////
    /**
     * Render a page...<p>
     */
    protected void showPage() {

        // get the object from the page list.
        Object obj = pages.get (activePage);
			
        // do we have a page reference?
        boolean haveRef = (obj instanceof String);
			
        // get a reference to the active page.
        active = (haveRef ? instanciatePage((String) obj) : (Page) obj);
			
        // check for valid page reference.
        if (active != null) {

            // remember the name of this page.
            lastPage = activePage;

            // set the page listener
            active.setPageListener (handler);
            
            // pass in the active EventQueue to the page.
            active.start (display, events);
        }
        // invalid or no page found.
        else
            // reset reference to pagename.
            activePage = null;
    }

    //#########################################################################
    /**
     * This method should return immediately to keep
     * the dispatcher from hanging.
     */
    public void startApp() {
        
        // activate an application page.
        showPage ();
    } 
    
    ///////////////////////////////////////////////////////////////////////////
    /**
     * suspend the page execution.<p>
     */
    public void pauseApp() {
    }

    ///////////////////////////////////////////////////////////////////////////
    /**
     * Destroy must cleanup everything.  The thread is signaled
     * to stop and no result is produced.<p>
     *
     * @param unconditional boolean - forced destroy of application?
     */
    public void destroyApp (boolean unconditional) {
    }
    
    ///////////////////////////////////////////////////////////////////////////
    // implementation of the data store interface.
    
    //-------------------------------------------------------------------------
    /**
     * set a named data object in the data store.<p>
     *
     * If there is no object associated with that name in
     * the data store, a new entry will be created and
     * linked to the data object.<p>
     * If there was a named object of the same name, it will
     * be released (overridden) by the new data object.<p>
     * The data object will be destroyed when the application
     * shuts down.<p>
     *
     *
     * @param key String - name of the data object.
     * @param value Object - data object.
     * @return boolean - successful operation?
     */
    public boolean setData (String key, Object value) {
        
        // key for persistent data?
        if (datastore.containsKey ("*" + key))
            // yes: failure.
            return false;
        
        // store the data.
        datastore.put (key, value);
        return true;
    }
    
    //-------------------------------------------------------------------------
    /**
     * add a named data object in the data store.<p>
     *
     * If there is no object associated with that name in
     * the data store, a new "persistent" entry will be created
     * and linked to the data object.<p>
     *
     * If there was a named object of the same name, the method
     * call will fail. It will also fail if the device does
     * not support some kind of persistence mechanism.<p>
     * The data object will *NOT* be destroyed when the application
     * shuts down but is stored persistently on the device.<p>
     *
     * @param key String - name of the data object.
     * @param value Object - data object.
     * @return boolean - successful operation?
     */
    public boolean addPersistentData (String key, Object value) {
        // generate the persistent key.
        String key2 = "*" + key;
        
        // key already there?
        if (datastore.containsKey (key) || datastore.containsKey (key2))
            // no: failure.
            return false;
        
        // put the data into the data store.
        datastore.put (key2, value);
        return true;
    }
    
    //-------------------------------------------------------------------------
    /**
     * drop a persistent data object.<p>
     *
     * @param key String - name of the object to be dropped.
     * @return boolean - successful operation?
     */
    public boolean dropPersistentData(String key) {
        // generate the persistent key.
        String key2 = "*" + key;
        
        // key available?
        if (!datastore.containsKey (key2))
            // no: report failure.
            return false;
        
        // drop the entry.        
        return (datastore.remove (key2) != null);
    }
    
    //-------------------------------------------------------------------------
    /**
     * get access to a named data object.<p>
     *
     * @param key String - name of the object to be retreived.
     * @return Object - data object from the store.
     */
    public Object getData (String key) {
        
        // try to get the transient data first
        Object res = datastore.get (key);
        if (res != null)
            // data available.
            return res;
        
        // try to get persistent data.
        return datastore.get ("*" + key);
    }
}
