// Source file: brf/j2me/midp/dynaworks/CanvasPage.java
// Subsystem: dynaworks
// Module: CanvasPage

// Copyright (C) 2000 by Bernd R. Fix.
// You may distribute this software under the terms of the Modified
// Artistic Licence, as specified in the accompanying LICENCE.TXT file.

package brf.j2me.midp.dynaworks;

// import external definitions.
import brf.j2me.midp.dynaworks.events.*;
import javax.microedition.lcdui.*;

/**
 * The CanvasPage class is the base class for Canvas-derived pages.<p>
 *
 * @author Bernd R. Fix
 * @version 1.0
 * @see brf.j2me.midp.dynaworks.Page
 */
public abstract class CanvasPage extends Page {

    ///////////////////////////////////////////////////////////////////////////
    /**
     * inner class for Canvas management. The event-related methods
     * generate appropriate Event objects that are pushed onto the
     * event queue. It is up to the handleEvent() method in a derived
     * class to process these events.
     */
    public class MyCanvas extends Canvas {

        //---------------------------------------------------------------------
        /**
         * key pressed.<p>
         *
         * @param keyCode int - button identifier.
         */
        protected void keyPressed  (int keyCode) {
            theQueue.push (new Event (Event.KEY_DOWN, keyCode));
        }
        /**
         * key repeated.<p>
         *
         * @param keyCode int - button identifier.
         */
        protected void keyRepeated (int keyCode) {
            theQueue.push (new Event (Event.KEY_REPEAT, keyCode));
        }
        /**
         * key released.<p>
         *
         * @param keyCode int - button identifier.
         */
        protected void keyReleased (int keyCode) {
            theQueue.push (new Event (Event.KEY_UP, keyCode));
        }

        //---------------------------------------------------------------------
        /**
         * pen touching the screen.<p>
         *
         * @param x int - x-coordinate of pen position.
         * @param y int - y-coordinate of pen position.
         */
        protected void pointerPressed  (int x, int y) {
            theQueue.push (new Event (Event.PEN_DOWN, x, y));
        }
        /**
         * pen moving across the screen.<p>
         *
         * @param x int - current x-coordinate of pen.
         * @param y int - current y-coordinate of pen.
         */
        protected void pointerDragged  (int x, int y) {
            theQueue.push (new Event (Event.PEN_MOVE, x, y));
        }
        /**
         * pen leaving the screen.<p>
         *
         * @param x int - x-coordinate of pen position.
         * @param y int - y-coordinate of pen position.
         */
        protected void pointerReleased (int x, int y) {
            theQueue.push (new Event (Event.PEN_UP, x, y));
        }

        //---------------------------------------------------------------------
        /**
         * repainting the screen.<p>
         *
         * @param g Graphics - canvas to be used.
         */
        protected void paint (Graphics g) {
            // call stub method of enclosing class.
            paintStub (g);
        }
    }
    /**
     * instance of the canvas class.<p>
     */    
    private MyCanvas canvas;
    
    ///////////////////////////////////////////////////////////////////////////
    /**
     * constructor.<p>
     *
     * @param label String - name of the page
     */
    public CanvasPage (String label) {
        // super messaging.
        super (label);
       
        // initialize canvas.
        canvas = new MyCanvas();
        // we are listening to our own commands.
        canvas.setCommandListener (this);
    }

    ///////////////////////////////////////////////////////////////////////////
    /**
     * add a command.<p>
     *
     * @param c Command - command object to be added.
     */
    public void addCommand (Command c) {
        canvas.addCommand (c);
    }
    
    ///////////////////////////////////////////////////////////////////////////
    /**
     * start the page thread.<p>
     *
     * @param display Display - display context to be used
     * @param queue EventQueue - event queue to be used
     */
    public void start (Display display, EventQueue queue) {
        // activate the canvas.
        display.setCurrent (canvas);
        // super messaging.
        super.start (display, queue);
    }
    
    ///////////////////////////////////////////////////////////////////////////
    /**
     * render the data.<p>
     *
     * @param g Graphics - graphic used for drawing.
     */
    public abstract void paint (Graphics g);
    /**
     * render stub: pass call on to "real" paint() method.<p>
     *
     * @param g Graphics - graphic used for drawing.
     */
    private final void paintStub (Graphics g) { paint (g); }  
    
    /**
     * force a repaint.
     */
    public void repaint () {
        canvas.repaint();
    }
    
    ///////////////////////////////////////////////////////////////////////////
    // getter/setter methods.
    /**
     * return the width of the display (in pixel)<p>
     *
     * @return int - number of pixels horizontally
     */
    public int getWidth () {
        return canvas.getWidth();
    }
    
    /**
     * return the height of the display (in pixel)<p>
     *
     * @return int - number of pixels vertically
     */
    public int getHeight () {
        return canvas.getHeight();
    }
}
