// Source file: brf/j2me/midp/dynaworks/Page.java
// Subsystem: dynaworks
// Module: Page

// Copyright (C) 2000 by Bernd R. Fix.
// You may distribute this software under the terms of the Modified
// Artistic Licence, as specified in the accompanying LICENCE.TXT file.

package brf.j2me.midp.dynaworks;

// import external definitions.
import brf.j2me.midp.dynaworks.events.*;
import javax.microedition.lcdui.*;

/**
 * The Page class is the abstract base class for displayable
 * pages (screens, forms, canvases or custom classes).<p>
 *
 * @author Bernd R. Fix
 * @version 1.0
 * @see brf.j2me.midp.dynaworks.Application
 */
public abstract class Page implements CommandListener,
                                      Runnable, DataStore {

    ///////////////////////////////////////////////////////////////////////////
    /**
     * name of the page.<p>
     */
    protected String name = null;
    /**
     * reference to the canvas.<p>
     */
    protected Display display = null;
    /**
     * name of next page.<p>
     */
    protected String nextPage = null;
    /**
     * page event listener reference.<p>
     */
    private PageListener listener = null;
    /**
     * reference to data store implementation.<p>
     */
    private DataStore data = null;
    
    ///////////////////////////////////////////////////////////////////////////
    /**
     * constructor.<p>
     *
     * @param label String - name of the page.
     */
    public Page (String label) {
        // remember our name
        name = label;
    }

    ///////////////////////////////////////////////////////////////////////////
    /**
     * set a new page event listener.<p>
     *
     * @param l PageListener - new listener of page events.
     */
    public void setPageListener (PageListener l) {
        listener = l;
    }

    ///////////////////////////////////////////////////////////////////////////
    /**
     * get the name of the page.<p>
     *
     * @return String - name of the page.
     */
    public String getName () {
        return name;
    }
    
    /**
     * set a reference to a data store.<p>
     *
     * @param ds DataStore - new data store to be used for variables.
     */
    public void setDataStore (DataStore ds) {
        data = ds;
    }

    ///////////////////////////////////////////////////////////////////////////
    /**
     * the page is running it's own task...
     */
    private Thread task = null;
    protected boolean running = false;
    protected EventQueue theQueue = null;

    //-------------------------------------------------------------------------
    /**
     * are there any events pending?<p>
     *
     * @return boolean - events pending.
     */
    public boolean eventsPending () {
        if (theQueue == null)
            return false;
        return theQueue.eventsPending();
    }

    //-------------------------------------------------------------------------
    /**
     * start the page thread.<p>
     *
     * @param display Display - display to be used for output.
     * @param queue EventQueue - event queue to be used.
     */
    public void start (Display display, EventQueue queue) {
              
        // set the event queue.
        theQueue = queue;
        
        // store reference to display.
        this.display = display;
        
        // run this page.
        running = true;
        task = new Thread (this);
        task.start ();
    }

    //-------------------------------------------------------------------------
    /**
     * page thread work method.<p>
     */
    public void run () {
        
        // while we are running...
        while (running) {
            
            // fair play.
            Thread.yield ();
            
            // next "tick" in the main task.
            tick ();

            // get next event in the queue.
            Event evt = theQueue.pop (false);
            
            // if we have no input event to process, ...
            if (evt == null)
                // we can perform some idle processing.
                evt = new Event (Event.APP_IDLE);
                
            // pass the event to the handler.
            handleEvent (evt);
        }

        // send a PageEvent to the listener about the shutdown.
        if (listener != null)
            listener.closing (nextPage);
    }
    
    //---------------------------------------------------------
    /**
     * perform a slice of the main task.<p>
     */
    public void tick () {
    }

    //---------------------------------------------------------
    /**
     * exit the page.
     */
    public void exit (String name) {
        // remember what page to call next.
        nextPage = name;
        // stop the worker thread.
        running = false;
    }

    //---------------------------------------------------------
    /**
     * handle an event.<p>
     *
     * @param e Event - event to be processed.
     * @return boolean - event handled?
     */
    public boolean handleEvent (Event e) {
        // is this a command event?
        if (e.getCode() == PageEvent.PAGE_CMD) {
            
            // get the command code.
            String cmd = ((PageEvent)e).getAction();
            
            //---------------------------------------------------------------------
            // 'about' default behaviour.
            if (cmd.compareTo ("About") == 0) {
                
                // perpare the 'About' box
	        Alert alert = new Alert ("dynaMID");
	        alert.setTimeout (Alert.FOREVER);

                // get the appropriate icon.
                String icon = "/icons/dm-2.png";
                if (display.numColors() > 2)
                    icon = display.isColor() ?                 
		        "/icons/dm-8.png" : "/icons/dm-4.png";

                // display the image.
	        try {
	            Image image = Image.createImage (icon);
		    alert.setImage(image);
	        } catch (java.io.IOException x) {
 		    // just don't append the image.
	        }
                
                // show the about box.
                display.setCurrent(alert);
                return true;     
            }
        }
        // event not handled.
        return false;
    }  
    
    ///////////////////////////////////////////////////////////////////////////
    /*
     * Respond to commands.<p>
     *
     * @param c Command - command sent
     * @param s Displayable - originating screen
     */
    public void commandAction (Command c, Displayable s) {
        theQueue.push (new PageEvent (PageEvent.PAGE_CMD, c.getLabel()));
    }
    
    ///////////////////////////////////////////////////////////////////////////
    // implementation of the data store interface.
    
    //-------------------------------------------------------------------------
    /**
     * set a named data object in the data store.<p>
     *
     * If there is no object associated with that name in
     * the data store, a new entry will be created and
     * linked to the data object.<p>
     * If there was a named object of the same name, it will
     * be released (overridden) by the new data object.<p>
     * The data object will be destroyed when the application
     * shuts down.<p>
     *
     *
     * @param key String - name of the data object.
     * @param value Object - data object.
     * @return boolean - successful operation?
     */
    public boolean setData (String key, Object value) {
        if (data == null)
            return false;
        return data.setData (key, value);
    }
    
    //-------------------------------------------------------------------------
    /**
     * add a named data object in the data store.<p>
     *
     * If there is no object associated with that name in
     * the data store, a new "persistent" entry will be created
     * and linked to the data object.<p>
     *
     * If there was a named object of the same name, the method
     * call will fail. It will also fail if the device does
     * not support some kind of persistence mechanism.<p>
     * The data object will *NOT* be destroyed when the application
     * shuts down but is stored persistently on the device.<p>
     *
     * @param key String - name of the data object.
     * @param value Object - data object.
     * @return boolean - successful operation?
     */
    public boolean addPersistentData(String key,Object value) {
        if (data == null)
            return false;
        return data.addPersistentData (key, value);
    }
    
    //-------------------------------------------------------------------------
    /**
     * drop a persistent data object.<p>
     *
     * @param key String - name of the object to be dropped.
     * @return boolean - successful operation?
     */
    public boolean dropPersistentData(String key) {
        if (data == null)
            return false;
        return data.dropPersistentData (key);
    }
    
    //-------------------------------------------------------------------------
    /**
     * get access to a named data object.<p>
     *
     * @param key String - name of the object to be retreived.
     * @return Object - data object from the store.
     */
    public Object getData(String key) {
        if (data == null)
            return null;
        return data.getData (key);
    }
}
