// Source file: brf/j2me/midp/dynaworks/events/EventQueue.java
// Subsystem: events
// Module: EventQueue

// Copyright (C) 2000 by Bernd R. Fix.
// You may distribute this software under the terms of the Modified
// Artistic Licence, as specified in the accompanying LICENCE.TXT file.

package brf.j2me.midp.dynaworks.events;

import java.util.Vector;


/**
 * The EventQueue represents a FIFO stack of events.<p>
 * 
 * An Event is appended to the EventQueue via the 'push()' method.
 * It is appended at the end of the queue and is processed after
 * all previous events have been handled.<p>
 * 
 * The next event from the EventQueue is retreived with the 'pop()'
 * method. The boolean argument to the method decides whether the
 * method should block until there is an Event available.<p>
 *  
 * When an application registers with the Environment, it passes its
 * EventQueue object to the environment. There the implementation
 * dependend event sources (system event loop) are linked to that
 * EventQueue. That ensures that all system and user interaction
 * events are passed to the EventQueue and can be processed by the
 * application.<p>
 *
 * @author Bernd R. Fix
 * @version 1.0
 * @see brf.j2me.midp.dynaworks.Application
 * @see brf.j2me.midp.dynaworks.events.Event
 */
public final class EventQueue {

    //----------------------------------------------------------------	
    /**
     * vector holding the events.<p>
     */
    private Vector queue;
    /**
     * semaphore for 'wait for event'.<p>
     */
    private Object semaphore;

    //----------------------------------------------------------------	
    /**
     * constructor.<p>
     */
    public EventQueue () {
        // initialize attributes.
        queue = new Vector (100);
        semaphore = new Object ();
    }
    
    //----------------------------------------------------------------	
    /**
     * push an event onto the event stack (queue = fifo stack).<p>
     *
     * @param e Event - event to be queued.
     * @return boolean - successful operation?
     */
    public boolean push (Event e) {
        
        // only one thread at a time...
        synchronized (queue) {	
            // append event at the end of vector.
            queue.addElement (e);
        	
            // if there is someone waiting for an event,
            // give that thread a 'go'.
            synchronized (semaphore) {
                semaphore.notify ();
            }
        }
        // report success.
        return true;
    }
    
    //----------------------------------------------------------------	
    /**
     * pop an event from the event stack (queue = fifo stack).<p>
     *
     * @param wait boolean - wait for an event in case of empty queue?
     * @return Event - next event to be processed (or null).
     */
    public Event pop (boolean wait) {
    	
        // is there any event in the queue?
        if (queue.size() == 0) {
        	
            // should we wait for an event?
            if (!wait)
                // no event available.
                return null;
                
            // wait for an event.
            try {
                synchronized (semaphore) {
                    semaphore.wait ();
                }
            } catch (InterruptedException e) {
                // if we are interrupted, no
                // event is available.
                return null;
            }
        }

        // only one thread at a time...
        Event e = null;
        synchronized (queue) {
        	
            // someone has stolen an event ?!
            // (probably due to a call to the 'reset()' method.
            if (queue.size() == 0)
                // no event available.
                return null;
            
            // get the next event out of the queue.
            e = (Event) queue.elementAt (0);
            // drop it in the queue.
            queue.removeElementAt (0);     	
        }
        // return the event.
        return e;
    }
    
    //----------------------------------------------------------------	
    /**
     * check for event waiting for processing.<p>
     *
     * @return boolean - event waiting?
     */
    public boolean eventsPending () {
        // are there any events?
        return (queue.size() > 0);
    }
    
    //----------------------------------------------------------------	
    /**
     * reset the EventQueue.<p>
     */
    public void reset () {
    
        // remove all events from the queue and unlock semaphore.
        queue.removeAllElements ();
        synchronized (semaphore) {
	      semaphore.notifyAll ();
        }
    }
}
